@@ -103,7 +103,6 @@ int main(int argc, char** argv)
     int *reqs, *sts, **buf;
     MPI_Offset  myNX, G_NX, myOff, block_start, block_len;
     vector<MPI_Offset> start(2), count(2);
-    int bb_enabled=0;
 
     MPI_Init(&argc, &argv);
     MPI_Comm_rank(MPI_COMM_WORLD, &rank);
@@ -127,18 +126,6 @@ int main(int argc, char** argv)
         NcmpiFile nc(MPI_COMM_WORLD, filename, NcmpiFile::replace,
                      NcmpiFile::classic5);
 
-    {
-        int flag;
-        char hint[MPI_MAX_INFO_VAL];
-        MPI_Info infoused;
-
-        nc.Inq_file_info(&infoused);
-        MPI_Info_get(infoused, (char*)"nc_burst_buf", MPI_MAX_INFO_VAL - 1, hint, &flag);
-        if (flag && strcasecmp(hint, "enable") == 0)
-            bb_enabled = 1;
-        MPI_Info_free(&infoused);
-    }
-
         /* the global array is NY * (NX * nprocs) */
         G_NX  = NX * nprocs;
         myOff = NX * rank;
@@ -166,9 +153,7 @@ int main(int argc, char** argv)
 
         free(buf[0]);
 
-        if (bb_enabled) {
-            nc.flush();
-        }
+        nc.flush();
 
         /* initialize the buffer with rank ID. Also make the case interesting,
            by allocating buffers separately */
