@@ -34,7 +34,7 @@
 
 /*----< ncmpio_free_NC_var() >-----------------------------------------------*/
 /* Free NC_var object */
-inline void
+void
 ncmpio_free_NC_var(NC_var *varp)
 {
     if (varp == NULL) return;
@@ -117,7 +117,7 @@ dup_NC_var(const NC_var *rvarp)
 /* vararray */
 
 /*----< ncmpio_free_NC_vararray() >------------------------------------------*/
-inline void
+void
 ncmpio_free_NC_vararray(NC_vararray *ncap)
 {
     int i;
@@ -603,9 +603,9 @@ err_check:
 
     if (! NC_indef(ncp)) { /* when file is in data mode */
         /* Let root write the entire header to the file. Note that we cannot
-         * just update the variable name in its space occupied in the file
-         * header, because if the file space occupied by the name shrinks, all
-         * the metadata following it must be moved ahead.
+         * just update the name in-place in file header, because if the file
+         * space occupied by the name shrinks, all the metadata following it
+         * must be moved ahead.
          */
         err = ncmpio_write_header(ncp);
         if (err != NC_NOERR) DEBUG_RETURN_ERROR(err)
