@@ -115,8 +115,8 @@ typedef struct PNC_driver PNC_driver;
 
 struct PNC_var {
     int         ndims;
-    int         recdim;
-    nc_type     xtype;
+    int         recdim;   /* if >=0, this is a record variable */
+    nc_type     xtype;    /* external NC data type */
     MPI_Offset *shape;    /* [ndims] */
 };
 typedef struct PNC_var PNC_var;
@@ -133,6 +133,7 @@ struct PNC {
     int                ndims;       /* number of dimensions defined */
     int                unlimdimid;  /* dim ID of NC_UNLIMITED */
     int                nvars;       /* number of variables */
+    int                nrec_vars;   /* number of record variables */
     struct PNC_var    *vars;        /* array of variable objects */
     void              *ncp;         /* pointer to driver internal object */
     struct PNC_driver *driver;
@@ -144,6 +145,10 @@ typedef struct PNC PNC;
 
 extern PNC_driver* ncmpio_inq_driver(void);
 
+extern PNC_driver* nc4io_inq_driver(void);
+
+extern PNC_driver* ncadios_inq_driver(void); 
+
 extern PNC_driver* ncfoo_inq_driver(void);
 
 extern PNC_driver* ncbbio_inq_driver(void);
