@@ -75,7 +75,7 @@ int main(int argc, char** argv)
     char filename[256];
     int i, rank, nprocs, err, nerrs=0;
     int ncid, cmode, varid[3], dimid[2], num_reqs, *buffer, *r_buffer;
-    MPI_Offset w_len, **starts, **counts;
+    MPI_Offset w_len, **starts=NULL, **counts=NULL;
 
     MPI_Init(&argc, &argv);
     MPI_Comm_rank(MPI_COMM_WORLD, &rank);
@@ -114,6 +114,10 @@ int main(int argc, char** argv)
     CHECK_ERR
     err = ncmpi_def_var(ncid, "var", NC_INT, NDIMS, dimid, &varid[0]);
     CHECK_ERR
+    if (nprocs < 4) { /* need 4 processes to fill the variables */
+        err = ncmpi_set_fill(ncid, NC_FILL, NULL);
+        CHECK_ERR
+    }
     err = ncmpi_enddef(ncid);
     CHECK_ERR
 
@@ -124,13 +128,15 @@ int main(int argc, char** argv)
     else if (rank == 2) num_reqs = 5;
     else if (rank == 3) num_reqs = 5;
 
-    starts    = (MPI_Offset**) malloc(num_reqs *        sizeof(MPI_Offset*));
-    counts    = (MPI_Offset**) malloc(num_reqs *        sizeof(MPI_Offset*));
-    starts[0] = (MPI_Offset*)  calloc(num_reqs * NDIMS, sizeof(MPI_Offset));
-    counts[0] = (MPI_Offset*)  calloc(num_reqs * NDIMS, sizeof(MPI_Offset));
-    for (i=1; i<num_reqs; i++) {
-        starts[i] = starts[i-1] + NDIMS;
-        counts[i] = counts[i-1] + NDIMS;
+    if (num_reqs > 0) {
+        starts    = (MPI_Offset**) malloc(num_reqs *        sizeof(MPI_Offset*));
+        counts    = (MPI_Offset**) malloc(num_reqs *        sizeof(MPI_Offset*));
+        starts[0] = (MPI_Offset*)  calloc(num_reqs * NDIMS, sizeof(MPI_Offset));
+        counts[0] = (MPI_Offset*)  calloc(num_reqs * NDIMS, sizeof(MPI_Offset));
+        for (i=1; i<num_reqs; i++) {
+            starts[i] = starts[i-1] + NDIMS;
+            counts[i] = counts[i-1] + NDIMS;
+        }
     }
 
     /* assign arbitrary starts and counts */
@@ -215,10 +221,12 @@ int main(int argc, char** argv)
 
     free(buffer);
     free(r_buffer);
-    free(starts[0]);
-    free(counts[0]);
-    free(starts);
-    free(counts);
+    if (num_reqs > 0) {
+        free(starts[0]);
+        free(counts[0]);
+        free(starts);
+        free(counts);
+    }
 
     /* check if PnetCDF freed all internal malloc */
     MPI_Offset malloc_size, sum_size;
